/* get_mrate is a program that calls mrate for a computation
on a single complete file; more complicated calling programs could
step through the data with overlapping windows, or else find
between-pause "spurts" (our current favorite), but this calling program
is intended as an example to show how mrate is called.
It returns an estimate of the syllable rate based on one of several
estimates (or their average, which appears to work best): correlation
between subband energy onsets, peaks in fullband energy, and
spectral moment of the fullband energy trajectory.
The default is the average of the three estimates.

Currently (11-24-97) the sampling rate is fixed at 8 kHz,
and the maximum data input length is 125 seconds of speech.

The estimate is put out in ascii form with a line feed.

Update by Oriol Vinyals 8/3/2013 : Removed some of the limitations and made the code more file friendly. 
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <sndfile.h>
#include "rate.h"
#include "functions.h"

#define DATBUFSIZE 10000000
#define BLOCK_SIZE 512

main(int argc, char **argv)
{
	/* data arrays */
        float *fdata;
        fdata = (float *) calloc(DATBUFSIZE,sizeof(float));

	float mrate_val; /* subband-based syllable onset count */
	float mrate (float *, int, int, int, int, int ); /* main function */
	float ftmp;
	float beg_time=0;
	float end_time=-1;

	int samprate;

	int winlength; /* window length in sample pts */
	
	int fullflag; /* compute fullband peak rate */
	int enflag; /* compute energy rate via spectral moment (enrate) */
	int mflag; /* compute multi-band correlation peak rate */

	int ntimes; /* number of times to run analysis (for timing
			purposes only) */

	int nread;
	int ascii_in;
        int sndfile_in;
	float samp;

	float wintime;

	int i, j;

	SNDFILE *infile = NULL;
	SF_INFO sfinfo;
	char * infilename;

	/* getopt variables */
	int c;
	extern int optind;
  	extern char *optarg;

	/* Defaults  (see rate.h) */

	ascii_in = TRUE;
	sndfile_in = FALSE;
	samprate = DEF_SAMPRATE;

	fullflag = FALSE;
	enflag = FALSE;
	mflag = FALSE;

	ntimes = 1;

	if(argc == 1)
	{
		printusage(argv[0]);
		exit(0);
	}

	/* Command line variable acquisition */
	while((c = getopt(argc, argv, "anfi:mt:b:e:")) != -1)
	{
		switch(c)
		{
			case 'a':
				ascii_in = TRUE;
				break;

			case 'b':
			  /* begining time */
			  beg_time = atof(optarg);
			  break;

		case 'e':
		  /*end time */
		  end_time = atof(optarg);
		  break;

			case 'f':
				/* run fullband analysis */
				fullflag = TRUE;
				break;

			case 'm':
				/* run multiband analysis */
				mflag = TRUE;
				break;

			case 'n':
				/* run enrate analysis */
				enflag = TRUE;
				break;

		 case 'i':
		   ascii_in = FALSE;
		   sndfile_in = TRUE;
		  infilename = optarg;
		  break;

			case 't':
				ntimes = atoi(optarg);
          			break;

			default:
 				printusage(argv[0]);
				break;
		}
	}
	/* If no flag used, set all flags to true (default is mean of 3) */
	if((fullflag==FALSE)&&(enflag==FALSE)&&(mflag==FALSE))
	{
		enflag = TRUE;
		mflag = TRUE;
		fullflag = TRUE;
	}


	/* Read in the raw data file */
	if(sndfile_in == TRUE)
	  {
	    if ((infile = sf_open (infilename, SFM_READ, &sfinfo)) == NULL)
	      {printf ("Not able to open input file %s.\n", infilename) ;
		puts (sf_strerror (NULL)) ;
		return 1 ;
	      } ;
	    samprate = sfinfo.samplerate;
	    if(end_time==-1){
	      end_time = (float) DATBUFSIZE/(float) samprate;
	    }
	    int readcount,k;
	    int end_frame,beg_frame;
	    beg_frame =(int)  (beg_time *  samprate);
	    end_frame = (int) (end_time *  samprate);
	    if(end_frame<beg_frame)
	      {
		printf("Error with -b and -e times\n");
		return 0;
	      }
	    float * buf = (float*) calloc(BLOCK_SIZE,sizeof(float));
	    // Offset the file first
	    sf_seek(infile, beg_frame, SEEK_SET);
	    i=0;
            while ((readcount = sf_readf_float (infile, buf, BLOCK_SIZE)) > 0 
		   && i < end_frame-beg_frame)
	      {for (k = 0 ; k < readcount && i < end_frame-beg_frame ; k++){
		  fdata[i++]=buf[k];
		}
	      }
	  }
        else
	  {
	i = 0;
	while(1) /* infinite loop to read in data */
	{
		if(ascii_in == TRUE)
		{
			nread = scanf("%f", &ftmp);
		}
		else
		{
		  printf("Must specify reading mode\n");
		  return 0;
		}
		if(nread != 1) /* break when done with input */
		{
			break;
		}

		fdata[i++] = ftmp;
		/* now loop back unless all data has been read */
	}
	  }
	winlength = i;

	/* Compute window length in seconds */
	wintime = (float) winlength / (float)samprate;
	
	for(i=0;i<winlength;i++)
	  {
	    //printf("%d: %e\n",i,fdata[i]);
	  }
	printf("File read, first sample: %e Num samples: %d Sample Rate: %d\n",fdata[0],winlength,sfinfo.samplerate);
	for(i=0; i<ntimes; i++)
	{
		mrate_val = mrate(fdata, winlength, samprate, 
			mflag, fullflag, enflag);
	}

	printf("%.2f\n", mrate_val);

	exit(0);
}


void printusage(char *fname)
{
  fprintf(stderr,"usage: %s [options]\n", fname );
  fprintf(stderr,"Where stdin and stdout are used,\n");
  fprintf(stderr,"output is always an ascii number,\n");
  fprintf(stderr,"at least one option is required, and\n");
  fprintf(stderr,"where options can be any of the following:\n");
  fprintf(stderr,"\t -a for input ascii rather than shorts [TRUE]\n");
  fprintf(stderr,"\t -b/-e for beginning and ending time to process (in seconds)\n");
  fprintf(stderr,"\t -i fname to read any file supported by libsndfile\n");
  fprintf(stderr,"\t\tNote that default input is ascii\n");
  fprintf(stderr,"\t -f use fullband peak-peaking [FALSE]\n");
  fprintf(stderr,"\t -m use multiband peak-picking [FALSE]\n");
  fprintf(stderr,"\t -e use energy rate from spectral moment (enrate)[FALSE]\n");
  fprintf(stderr,"\t Computes the average of all measures chosen \n");
  fprintf(stderr,"\t Default (no rate alg specified) is mean of all 3 \n");
  fprintf(stderr,"\t -t number of times to run routine for timing [1]\n");
  exit( 0 );
}


