/*
mrate is a function that computes putative syllable
onset starts based on up to three different
measures: a multiband correlation function of the signal energy, 
using a peak-picking routine; a fullband version; and "enrate",
a fullband measure that is the spectral moment for the energy trajectory.
The result returned is the average of all estimate types that have flags set.

For the case of the multiband measure, the subband filters are
currently fixed, and are only correct for a sampling rate of 8 kHz;
a later version should make this adjustable. The resulting
time series are each half-wave rectified, filtered with a 16 Hz lowpass
FIR filter and downsampled to 100 Hz. 
The resulting subband energy is compressed with a cube root to estimate
a trajectory within a portion of the modulation
spectrum that corresponds to syllabic rates over a wide range
of speaking rates. For each point in this trajectory, we compute
the average value over all the pairs of subbands. 
In the fullband case we simply get a lowpass-filtered
energy trajectory to do the counting on. In either case we then apply
a peak-counting routine to this new trajectory and normalize by
the time in the speech chunk to get a rate value.
 
For the enrate component, any segment shorter than 1 second is
symmetrically padded with zeros to make it one second long.

This version is explicitly offline; that is, it computes the mrate
for the complete chunk of data that it is fed.

*/

#include <stdio.h>
#include <math.h>
#include "rate.h"
#include "functions.h"
#include "functions_en.h"

#include "bpf.h"

float mrate (float *fdata, int winlength, int samprate, 
	int mflag, int fullflag, int enflag )
{

/*********Declarations and Initializations ***********************/

	/* data arrays */

	static float *bpfdata;
	static float *dwndata;
	static float **smoothdata;
	static float *covar_out;

	static int first = TRUE;

	float wintime; /* length of analysis window in seconds */
	float mrate_val; /* estimate of multiband rate */

	float ftmp;

	int n_onsets;
	int n_measures;

	float downby;

	float nth_root; /* n for nth root compression */

	int i, j;

	int ascii_num;
	int show_covar = TRUE;
	int show_env = FALSE;

        int dwnwinlength; /* window length in downsampled points */

	int startfreq = 2; /* Start bin for spectral moments */

	short sval;

	char fname[256];
	char *basef;

	FILE *fp;

	/* getopt variables */
	int c;
	extern int optind;
  	extern char *optarg;

	basef = "tstout";

/************* Range check for input parameters *********/

	if(samprate < MIN_SAMPRATE)
	{
		fprintf(stderr,"samprate less than %d \n", MIN_SAMPRATE);
		exit(-1);
	}

	if(samprate > MAX_SAMPRATE)
	{
		fprintf(stderr,"samprate more than %d \n", MAX_SAMPRATE);
		exit(-1);
	}

	if(winlength < (.25 * samprate))
	{
		fprintf(stderr,"Warning: mrate running on %d samples \n",
			winlength);
	}


	if(( mflag != TRUE)&&( fullflag != TRUE)&&( enflag != TRUE))
	{
		fprintf(stderr,"Need to choose at least one estimate type\n");
		exit(-1);
	}

/********** Minor calculations *************/

	/* window length in seconds */
	wintime = (double)winlength / (double)samprate;

	/* downsampling factor for energy trajectory */
	downby = samprate / LOWRATE;

	/* length of downsampled window */
	dwnwinlength = (int)ceil((double)winlength / downby);

	nth_root = 3.0; /* Compression - cube root appears to be best ... */

/********** Allocations for arrays  *************/

	if(first != TRUE) free(bpfdata);
        bpfdata = (float *)malloc(winlength*sizeof(float));
        if(bpfdata == (float *)NULL)
        {
               fprintf(stderr,"Can't allocate mem for bpfdata\n");
               exit(-1);
        }

	if(first != TRUE) free(dwndata);
        dwndata = (float *)malloc(dwnwinlength*sizeof(float));
        if(dwndata == (float *)NULL)
        {
               fprintf(stderr,"Can't allocate mem for dwndata\n");
               exit(-1);
        }

/********** Multiband allocations  *************/

	if(mflag == TRUE)
	{
	      if(first != TRUE) free(covar_out);
	      covar_out = (float *)malloc(dwnwinlength*sizeof(float));
	      if(covar_out == (float *)NULL)
	      {
		     fprintf(stderr,"Can't calloc mem for ");
		     fprintf(stderr,"covar_out\n");
			   exit(-1);
	      }
	      if(first==TRUE) smoothdata = (float **)malloc(NBANDS*sizeof(float*));
	      for(i=0; i<NBANDS; i++)
	      {
		    if(first != TRUE) free(smoothdata[i]);
		    smoothdata[i] = (float *)calloc(dwnwinlength,
					sizeof(float));
		    if(smoothdata[i] == (float *)NULL)
		    {
			   fprintf(stderr,"Can't calloc mem for ");
			   fprintf(stderr,"smoothdata[%d]\n",i);
			   exit(-1);
		    }
	      }
	}

/************* Signal processing ******************/
	mrate_val = 0.;
	n_measures = 0;
// printf("Starting Signal Processing\n");fflush(stdout);
/* First the full band version ... */
	if(fullflag == TRUE)
	{
		/* copy the data */
		vecmove(bpfdata, fdata, winlength);
		/* rectify, filter, downsample */
       		envelope(bpfdata, dwndata, winlength, downby);
		n_onsets = getpeakonsets(dwndata, dwnwinlength);
		if(n_onsets < 1) n_onsets = 1; /* We assume at
						least 1 syllable ...) */
		
		mrate_val += (float)n_onsets / wintime;
		n_measures++;
	}
// printf("Multiband Version\n");fflush(stdout);
/* Now the multi-band version ... */
	if(mflag == TRUE)
	{


	/* Band pass filter into NBANDS channels (typ. 4). For each
	channel, compute a lowpass energy envelope, compress, normalize,
	and compute average cross-band products for each time point. */

	    for(i=0; i<NBANDS; i++)
	    {
	     // printf("firfilt\n");fflush(stdout);
		/* filter with FIR bandpass */
		runfirfilt(fdata, bpfdata, Bpfilt[i], 
			winlength, Bpfiltlength[i], 1 );
	// printf("envelope\n");fflush(stdout);
		/* rectify, filter, downsample */
        	envelope(bpfdata, dwndata, winlength, downby);
	// printf("compress\n");fflush(stdout);
		/* take the nth root */
		compress(dwndata, dwnwinlength, nth_root);
	// printf("vecmove\n");fflush(stdout);
		/* move the data */
		vecmove(smoothdata[i], dwndata, dwnwinlength);
	// printf("mean_div\n");fflush(stdout);
		/* divide by mean (so that average energy is 1) */
		mean_div(smoothdata[i], dwnwinlength);

                if(show_env )
                {
                        strcpy(fname, basef);
                        fname[6] = 49 + i;
                        fname[7] = 0;
                        fp = fopen(fname,"w");
                        for(j=0; j<dwnwinlength;j++)
                        {
                                fprintf(fp,"%g\n",smoothdata[i][j]);
                        }
                        fclose(fp);
                }
	    }
	    covar_pairs(smoothdata, NBANDS, dwnwinlength, covar_out);

	    if(show_covar)
	    {
		fp = fopen("tstoutcov","w");
		for(j=0; j<dwnwinlength;j++)
		{
			fprintf(fp,"%g\n", covar_out[j]);
		}
	    }

		/* For the covariance waveform, compute and 
		apply thresholds to determine putative onsets.
		Put a 1 in the indicator array when these occur,
		a zero everywhere else. This array is sometimes
		useful for diagnostics ... */

	    n_onsets = getpeakonsets(covar_out, dwnwinlength);
 	    if(n_onsets < 1) n_onsets = 1; /* We assume at
						least 1 syllable ...) */
	    mrate_val += (float)n_onsets / wintime;
	    n_measures++;
	}
// printf("Enrate Estimate\n");fflush(stdout);
/* Finally, the enrate estimate. Although there is some redundancy
between the pieces of this routine and those used in earlier functions,
enrate is used pretty much as previously designed, since that had been
debugged and tested ... */
	if(enflag == TRUE)
	{
		mrate_val += enrate(fdata, winlength, samprate);
		n_measures++;
	}

	mrate_val /= (float)n_measures;

	first = FALSE;

	return(mrate_val);
}
